#April 9,2019
##Max PM2.5 and average wind speed in 21 sites/stations in Canada in 2012
#skewed N-distribution: Measurement Error approach
#Model: yi=beta0+beta1*wi+beta2*xi+ei 

rm(list=ls(all=TRUE))

require(rjags)
library(dclone)
library(R2WinBUGS)


set.seed(432104)
#--------------------------
model<-function()
{
  for (i in 1:m) {
    z[i] <- ifelse(y[i] >= (beta0+beta1*W[i]+beta2*X[i]), (y[i] - beta0-beta1*W[i]-beta2*X[i])/gama, (y[i]-beta0-beta1*W[i]-beta2*X[i])*gama)
    loglik[i] <- logdensity.t(z[i], 0, tau, 1000) + log(2) - log(gama + 1/gama)
    lambda[i] <- C - loglik[i]
    zero[i] ~ dpois(lambda[i])
  
    #x[i] ~ dnorm(Wbar,0.000001)
    W[i] ~ dnorm(Wbar,1/varu) #assumed fixed to aviod identifiability issue (got from Fuller analysis)
  }
  
  varu~dnorm(12.94,0.0000001) #12.94 is the variance of monthly wind speed in 21 sites 
  
  #Priors on the model parameters
  #mu.x~dnorm(mux0,1)
  #mux0~dnorm(Wbar,0.01) %_% I(0, ) #see Rodrigues and Bolfarine (2007) 
  
 # sigmax ~ dunif(0,1000)
#  sigma2x<-pow(sigmax,2)
#  taux<-1/sigma2x
  
  tau~dgamma(0.5,0.5)
 
  beta0 ~ dunif(-100000,100000) #dnorm(0,0.001)
  beta1 ~ dunif(-100000,100000) #dnorm(0,0.001) 
  beta2 ~ dunif(-100000,100000) #dnorm(0,0.001) 
  
  #Deterministic transformations
  sigma2<-1/tau
 # sigma2u<-1/tauu  
  
  gamma2~dgamma(0.5,0.5)
  gama<-pow(gamma2,0.5)
 
  #v ~ dgamma(1,0.1)
  }
#########################
#Data:

y<-c(39.63,11.94,8.33,13.06,33.89,29.49,84.4,27.59,50.88,22.78,
     25.32,35.79,51.16,14.54,12.62,17.73,87.55,30.97,48.06,13.61,
     34.86) #Max PM2.5

#Temperature (0 C) when PM2.5 is max. (error free covariate)
X<-c(26.10,19.00,-20.00,15.10,23.00,21.80,19.40,29.20,18.80,24.50,-11.60,0.00,-12.70,-1.20,
      -12.90,18.90,-25.70,-8.30,-2.70,14.40,-9.70)

#Wind speed covariate (Measuremtn error covariate) 
W<-c(17.464706, 13.711765, 15.929412,  9.164706, 14.150000, 16.164706, 11.264706, 15.770588,
     14.805882,14.623529, 14.629412, 13.864706, 14.888235, 15.188235, 12.088235, 17.476471,
     17.229412,  8.129412,6.441176 ,10.047059, 11.676471) #Average of wind speed

Wbar<-mean(W)
m=21
zero<-rep(1,m)
C <- 10000


parameters.hb <- c("beta0", "beta1", "beta2", "sigma2", "gama") 
                    #---------------------------
                    #Initial values
                #    inits<-function(){
                #    list(
                #    beta0=50, beta1=0.6,tau=0.02,gamma2=1, v=10)
                #    }
                    #--------------------------

data<-list(y=y,W=W,X=X,m=m,zero=zero, C=C,Wbar=Wbar) 
             model.out<- jags.fit(data=data,parameters.hb, model,n.chains=2,thin = 5, n.iter=50000, n.adapt=3000, n.update=10000, DIC=T) #, inits=inits)
          
             update(updated.model(model.out),20000)
             modelout.mcmc<-coda.samples(updated.model(model.out),parameters.hb,n.iter=5000,thin=5)
             num<-0
             while( (max ((gelman.diag(modelout.mcmc))$psrf[,1]) >1.05) & num<20){
               update(updated.model(model.out),20000)
               modelout.mcmc<-coda.samples(updated.model(model.out),parameters.hb,n.iter=5000,thin=5)
               num<-num+1
             }
             num
             #0    
            max((gelman.diag(modelout.mcmc))$psrf[,1]) 
             #     [1] 1.039197
             ((gelman.diag(modelout.mcmc))$psrf[,1]) 
          #  beta0    beta1    beta2     gama   sigma2 
          #  1.039197 1.025119 1.005274 1.007754 1.003006 
             
             summary(modelout.mcmc)
             
         #    Iterations = 83005:88000
        #     Thinning interval = 5 
        #     Number of chains = 2 
        #     Sample size per chain = 1000 
             
         #    1. Empirical mean and standard deviation for each variable,
          #   plus standard error of the mean:
               
          #     Mean       SD Naive SE Time-series SE
          #   beta0   13.65105  20.5267 0.458990       1.917208
          #   beta1    0.28379   1.4329 0.032040       0.132190
          #   beta2    0.01248   0.2488 0.005562       0.006993
          #   gama     1.91770   0.5631 0.012592       0.025290
          #   sigma2 301.81139 158.7947 3.550757       6.442581
             
         #    2. Quantiles for each variable:
               
          #     2.5%      25%       50%      75%    97.5%
          #   beta0  -28.4534   1.8650  14.45201  26.1818  52.9455
          #   beta1   -2.4024  -0.6689   0.21108   1.1688   3.3515
          #   beta2   -0.5252  -0.1332   0.02378   0.1764   0.4673
          #   gama     0.9963   1.5052   1.85756   2.2663   3.1595
          #   sigma2  88.9580 185.8830 273.91579 380.9986 703.4679
###################################################################  
  #Write the model in such a way to get the DIC from jags function:
             
             model1<-"model{
               for (i in 1:m) {
                 z[i] <- ifelse(y[i] >= (beta0+beta1*W[i]+beta2*X[i]), (y[i] - beta0-beta1*W[i]-beta2*X[i])/gama, (y[i]-beta0-beta1*W[i]-beta2*X[i])*gama)
                 loglik[i] <- logdensity.t(z[i], 0, tau, 1000) + log(2) - log(gama + 1/gama)
                 lambda[i] <- C - loglik[i]
                 zero[i] ~ dpois(lambda[i])
                 
                 W[i] ~ dnorm(Wbar,1/varu) #assumed fixed to aviod identifiability issue (got from Fuller analysis)
               }
               
               varu~dnorm(12.94,0.0000001) #12.94 is the variance of monthly wind speed in 21 sites 
               
               #Priors on the model parameters
               
               tau~dgamma(0.5,0.5)
               
               beta0 ~ dunif(-100000,100000) #dnorm(0,0.001)
              beta1 ~ dunif(-100000,100000) #dnorm(0,0.001) 
             beta2 ~ dunif(-100000,100000) #dnorm(0,0.001) 
               
               #Deterministic transformations
               sigma2<-1/tau
               gamma2~dgamma(0.5,0.5)
               gama<-pow(gamma2,0.5)
              # v ~ dgamma(1,0.1)
             }"
             ######################### 
             model11 <- jags.model(textConnection(model1),data = list(y=y,W=W,X=X,m=m,zero=zero, C=C,Wbar=Wbar),n.chains=3)     
             dIc<-dic.samples(model11, n.chains=3,thin = 5, n.iter=50000, n.adapt=3000, n.update=10000) 
             dIc
           #  Mean deviance:  419907 
          #   penalty 1.244 
          #   Penalized deviance: 419908 
             ###########################################################################
             