
#The following R code was written for the performance of bias and SE of the Skewed T-distrubution (without ME) in the LM; note that data generated from skewed-T with ME

date()
set.seed(1980)
path<-getwd()
library("rjags")
library("dclone")
library("sn")
library("skewt")

#
R=200
s1=100
#=============================================
#Define the constants of the data and model

model<-function()
{
  for (i in 1:m) {
    z[i] <- ifelse(y[i] >= (beta0+beta1*x[i]), (y[i] - beta0-beta1*x[i])/gama, (y[i]-beta0-beta1*x[i])*gama)
    loglik[i] <- logdensity.t(z[i], 0, tau, v) + log(2) - log(gama + 1/gama)
    lambda[i] <- C - loglik[i]
    zero[i] ~ dpois(lambda[i])
    
    # W[i] ~ dnorm(x[i],1/3)
    # U[i] ~ dnorm(0,1/3)
    # xx[i]<-W[i]-U[i]
  }
  
  #Priors on the model parameters
  #mu.x~dnorm(0,0.0001)
  
  
  tau~dgamma(0.5,0.5) 
  #tauu~dgamma((60*60/0.01)+2,((60*60*60/0.01)+60))
  #taux~dgamma(2,500) #(0.001,0.001)
  
  beta0 ~ dunif(-100000,100000) #dnorm(0,0.001)
  beta1 ~ dunif(-100000,100000) #dnorm(0,0.001) 
  
  
  #Deterministic transformations
  #sigma ~ dunif(0,100)
  #sigma2<-pow(sigma,2)
  #tau<-1/sigma2
  
  sigma2<-1/tau
  #sigma2x<-1/taux
  #  sigma2u<-1/tauu  
  
  gamma2 ~ dgamma(0.5,1)
  gama<-pow(gamma2,0.5)
  
  v ~ dgamma(1,0.2)
  
}
#-------------------------------------
m=50
zero<-rep(1,m)
C <- 10000

beta0=1
beta1=3
sigma2=1
sigma2u=1
sigma2x=9
mu.x=5
v=5
gama=0.7
#--------------------------------------

parameters.hb <- c("beta0", "beta1", "gama",  "sigma2", "v") 
#---------------------------
#Initial values
inits<-function(){
  list(
    beta0=1, beta1=3, tau=1, gamma2=0.5, v=5)
}
#--------------------------
#-------------------------------------------------
statstotal.hb<-matrix(0,1000,5)
colnames(statstotal.hb)<-c("beta0", "beta1", "gama",  "sigma2", "v")
stats.hb<-list()
modelout.sim<-modelout.mcmc<-list()
mean.hb<-MSE.hb<-sd.hb<-Bias.hb<-c(rep(0,5))
covp99.hb<-covp98.hb<-covp95.hb<-covp90.hb<-c(rep(0,5))  
domain99.hb<-domain98.hb<-domain95.hb<-domain90.hb<-0

paramvalue.hb<-c(rep(0,5))
outbound.hb<-outconv.hb<-rep(0,R)

#tem.seq<-seq(50,150,0.1)  #we expect the outcomes generated to be between 50 and 150 based on the parameters defined in the model.

x<-rnorm(m,mu.x,sqrt(sigma2x))


n=1
for(n in 1:R){
  y<-rep(0,m)
  W<-rep(0,m)
  random1<-rnorm(m,0,1)
  re<-rskt(m, v, gama)
  
  for(i in 1:m){
    W[i]<-x[i]+sqrt(sigma2u)*random1[i]  #mu.x+sqrt(sigma2x)*random1[i]+sqrt(sigma2u)*random2[i] #rnorm(1,mu.x,sqrt(sigma2x+sigma2u))
    y[i]<-beta0+beta1*W[i]+sqrt(sigma2)*re[i]
  }
  
  
  #
  # integrand1 <- function(y) {
  #zi<- ifelse(y >= (beta0+beta1*W[i]), 1/gama^2, gama^2)
  #return( (2*factorial(v/2-1/2)/( (gama+1/gama)*factorial(v/2-1)*sqrt(pi*v*sigma2) ) )*((1+ ((y- (beta0+beta1*W[i]))^2/(v*sigma2))*zi )^(-v/2-1/2)) )
  #                             }
  
  #tem.unif<-runif(1,0,1)
  
  #cum.y<-rep(0,1001)
  #for(j in 1:1001){
  #cum.y[j]<-as.numeric(integrate(integrand1, lower = -Inf, upper = tem.seq[j])[1])
  #              }
  
  #dif<-rep(0,1001)
  #for(j in 1:1001){
  #dif[j]<-abs(cum.y[j]-tem.unif)
  #              }
  #y[i]<-tem.seq[which.min(dif)]
  
  ##y[i]<-rst(1, xi=beta0+beta1*x[i], omega=sigma2, alpha=(gama-1), nu=v)
  #             }
  
  Wbar<-mean(W)
  print(summary(y))
  
  #Using jags.fit:
  #----------------------------------------------------HB model
  if(n>s1){
    data.sim<-list(y=y,m=m, zero=zero, C=C, x=x) 
    print(paste("---------------- HB model",n,"------------"))
    model.out<- jags.fit(data=data.sim, parameters.hb, model, n.chains=2, thin = 5, n.iter=50000, n.adapt=3000, n.update=10000, inits=inits)
    update(updated.model(model.out),20000)
    modelout.mcmc<-coda.samples(updated.model(model.out),parameters.hb,n.iter=5000,thin=5)
    num<-0
    while( (max ((gelman.diag(modelout.mcmc))$psrf[,1]) >1.05) & num<20){
      update(updated.model(model.out),20000)
      modelout.mcmc<-coda.samples(updated.model(model.out),parameters.hb,n.iter=5000,thin=5)
      num<-num+1
    }
    print(paste("model",n))
    print(paste("num",num))
    print(paste("Gelman"))
    print(((gelman.diag(modelout.mcmc))))
    stats.hb<-summary(modelout.mcmc, quantiles=c(0.005,0.01,0.025,0.05,0.95,0.975,0.99,0.995))
    print(stats.hb)
    outbound.hb[n]<-0
    if(max(abs((stats.hb[[1]][c((1):(5)),1])-c(beta0,beta1, gama,  sigma2,  v )))>4){
      outbound.hb[n]<-1
    }
    print(paste("outbound for HB",n, "is", outbound.hb[n]))
    
    
    outconv.hb[n]<-0
    if(max((gelman.diag(modelout.mcmc))$psrf[,1])>1.40){
      outconv.hb[n]<-1
    }
    print(paste("outconv for HB",n, "is", outconv.hb[n]))
    
    if(outconv.hb[n]==0){
      statstotal.hb[n,]<-stats.hb[[1]][,1]
      #####--------------------------------------------------------------------------------- 
      paramvalue.hb<-c(beta0,beta1, gama, sigma2, v) #mu.x,sigma2x, 
      print(paste("RB, MSE for model",n))
      mean.hb<-mean.hb+stats.hb[[1]][,1]
      covp99.hb<-covp99.hb+1*((stats.hb[[2]][,1]<paramvalue.hb & paramvalue.hb<stats.hb[[2]][,8] ))
      covp98.hb<-covp98.hb+1*((stats.hb[[2]][,2]<paramvalue.hb & paramvalue.hb<stats.hb[[2]][,7] ))
      covp95.hb<-covp95.hb+1*((stats.hb[[2]][,3]<paramvalue.hb & paramvalue.hb<stats.hb[[2]][,6] ))
      covp90.hb<-covp90.hb+1*((stats.hb[[2]][,4]<paramvalue.hb & paramvalue.hb<stats.hb[[2]][,5] ))
      domain99.hb<-domain99.hb+((stats.hb[[2]][,8]-stats.hb[[2]][,1] ))
      domain98.hb<-domain98.hb+((stats.hb[[2]][,7]-stats.hb[[2]][,2] ))
      domain95.hb<-domain95.hb+((stats.hb[[2]][,6]-stats.hb[[2]][,3] ))
      domain90.hb<-domain90.hb+((stats.hb[[2]][,5]-stats.hb[[2]][,4] ))
      
      MSE.hb<-MSE.hb+((stats.hb[[1]][,1]-paramvalue.hb)^2)
      Bias.hb<-Bias.hb+sqrt((stats.hb[[1]][,1]-paramvalue.hb)^2)
      sd.hb<-sd.hb+(stats.hb[[1]][,2])
      
      print(paste("------------ Bias for HB",n))
      print(Bias.hb)
      print(paste("------------ MSE for HB",n))
      print(MSE.hb)
      print(paste("------------ sd for HB",n))
      print(sd.hb)
      print(paste("------------ mean nodes for HB",n))
      print(mean.hb)
      print(paste("------------ coverage probability 99% for HB",n))
      print(covp99.hb)
      print(paste("------------ coverage probability 98% for HB",n))
      print(covp98.hb)
      print(paste("------------ coverage probability 95% for HB",n))
      print(covp95.hb)
      print(paste("------------ coverage probability 90% for HB",n))
      print(covp90.hb)
      print(paste("------------ domain for 99% for HB",n))
      print(domain99.hb)
      print(paste("------------ domain for 98% for HB",n))
      print(domain98.hb)
      print(paste("------------ domain for 95% for HB",n))
      print(domain95.hb)
      print(paste("------------ domain for 90% for HB",n))
      print(domain90.hb)
      print(paste("-------------------------------------------------------------------"))
    }
  }
}
#}
#  rm(a123)
#  stats<-list()
#  for(l in 1:R){
#  load(file=paste(path,"stats",l,".RData",sep=""))
#  stats[[l]]<-a123
#  rm(a123)}
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("############## final results for",R,"#####################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("##################################################"))
print(paste("                 "))
print(paste("############## HB",R,"#####################"))

print(paste("outbound for HB are:"))
print(outbound.hb[s1:R])
print(paste("sum of outbound for HB are:"))
print(sum(outbound.hb[s1:R]))
print(paste("------------ aggregated mean nodes for HB"))
print(mean.hb)
print(paste("------------ aggregated Bias for HB"))
print(Bias.hb)
print(paste("------------ aggregated MSE for HB"))
print(MSE.hb)
print(paste("------------ aggregated sd for HB"))
print(sd.hb)
print(paste("------------ coverage probability 99% for HB"))
print(covp99.hb)
print(paste("------------ coverage probability 98% for HB"))
print(covp98.hb)
print(paste("------------ coverage probability 95% for HB"))
print(covp95.hb)
print(paste("------------ coverage probability 90% for HB"))
print(covp90.hb)
#-------------------------------------
print(paste("------------ aggregated coverage probability 99% for HB "))
print(sum(covp99.hb[1:5]))
print(paste("------------ aggregated coverage probability 98% for HB "))
print(sum(covp98.hb[1:5]))
print(paste("------------ aggregated coverage probability 95% for HB "))
print(sum(covp95.hb[1:5]))
print(paste("------------ aggregated coverage probability 90% for HB "))
print(sum(covp90.hb[1:5]))
print(paste("------------ aggregated  MSE for HB "))
print(sum(MSE.hb[1:5]))
print(paste("------------ aggregated  sd for HB "))
print(sum(sd.hb[1:5]))
print(paste("------------ domain for 99% for HB"))
print(domain99.hb)
print(paste("------------ domain for 98% for HB"))
print(domain98.hb)
print(paste("------------ domain for 95% for HB"))
print(domain95.hb)
print(paste("------------ domain for 90% for HB"))
print(domain90.hb)
print(paste("------------ aggregated domain for 99% for HB"))
print(sum(domain99.hb[1:5]))
print(paste("------------ aggregated domain for 98% for HB"))
print(sum(domain98.hb[1:5]))
print(paste("------------ aggregated domain for 95% for HB"))
print(sum(domain95.hb[1:5]))
print(paste("------------ aggregated domain for 90% for HB"))
print(sum(domain90.hb[1:5]))


print(paste("##################################################"))
print(paste("                 "))


save(list = ls(all=TRUE), file = ".RData")

date()         
